//=============================================================================
// Project:    SLP - An Interpreter for Super Logic Programs / Static Semantics
// Module:     rvar.cxx
// Purpose:    Information about variable in the current rule
// Last Change:27.09.2001
// Language:   C++
// Author:     Stefan Brass
// Email:      Stefan.Brass@informatik.uni-giessen.de, sbrass@sis.pitt.edu
// Address:    Universitaet Giessen, Arndtstr. 2, 35392 Giessen, Germany
// Copyright:  (c) 2000-2001 by Stefan Brass
// Licence:    Distributed under the conditions of the GNU Public Licence.
//=============================================================================

//=============================================================================
// Interface:
//=============================================================================

#include "ver.h"
#include "out.h"
#include "var.h"

#include "rvar.h"

//=============================================================================
// Exported Functions:
//=============================================================================

//-----------------------------------------------------------------------------
// Constructor:
//-----------------------------------------------------------------------------

rvar_c::rvar_c(var_t var, int offset)
{
	// Check input parameters:
	CHECK(var != 0, "rvar_c::Constructor: variable is NIL");
	CHECK(var->valid(), "rvar_c::Constructor: variable is invalid");
	CHECK(offset >= 0, "rvar_c::Constructor: offset is negative");
	CHECK(offset < VER_MAX_VARS, "rvar_c::Constructor: offset too large");

	// Initialize Magic Number:
#ifdef VER_DEBUG
	Magic = RVAR_MAGIC;
#endif

	// Initialize Member Variables:
	Var = var;
	Offset = offset;
	Bound = BOOL_FALSE;
	Next = 0;
}

//-----------------------------------------------------------------------------
// Destructor:
//-----------------------------------------------------------------------------

rvar_c::~rvar_c(void)
{
#ifdef VER_DEBUG
	Magic = 0;
#endif
}

//-----------------------------------------------------------------------------
// Debugging Support (Integrity Check):
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
bool_t	rvar_c::valid(void) const
{
	// Check object pointer:
	CHECK(this != 0, "rvar_c::valid: NIL object");

	// The magic number must be right:
	if(Magic != RVAR_MAGIC)
		return(BOOL_FALSE);

	// The variable must be set:
	if(Var == 0)
		return(BOOL_FALSE);

	// The offset must be non-negative:
	if(Offset < 0)
		return(BOOL_FALSE);

	// The offset must be not too large:
	if(Offset >= VER_MAX_VARS)
		return(BOOL_FALSE);

	// The binding status must be a legal boolean value:
	if(Bound != BOOL_FALSE && Bound != BOOL_TRUE)
		return(BOOL_FALSE);

	// Ok:
	return(BOOL_TRUE);
}
#endif

//-----------------------------------------------------------------------------
// print: Print this variable (with offset).
//-----------------------------------------------------------------------------

void rvar_c::print(void) const
{
	// Check consistency:
	CHECK(valid(), "rvar_c::print: Bad object");

	// Print variable name:
	out_c::print_str(Var->name());

	// Print offset:
	out_c::print_char('[');
	out_c::print_int(Offset);
	out_c::print_char(']');
}
